package com.isu.gaswellwatch.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.isu.gaswellwatch.annotation.OperationLog;
import com.isu.gaswellwatch.dto.BlockCreateRequest;
import com.isu.gaswellwatch.dto.BlockEditRequest;
import com.isu.gaswellwatch.entity.Response;
import com.isu.gaswellwatch.enums.LogType;
import com.isu.gaswellwatch.service.BlockService;
import com.isu.gaswellwatch.vo.BlockVO;
import jakarta.validation.Valid;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 区块管理
 *
 * @author scwsl
 * @date 2024-11-17
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/blocks")
public class BlockController {

    private final BlockService blockService;

    /**
     * 查询区块列表
     */
    @GetMapping(value = "/page")
    public Response<Page<BlockVO>> page(@RequestParam(defaultValue = "1") Integer currentPage,
                                        @RequestParam(defaultValue = "10") Integer pageSize,
                                        @RequestParam(required = false) String name) {
        return Response.succeed(this.blockService.pageForQuery(currentPage, pageSize, name));
    }

    /**
     * 获取区块详细信息
     */
    @GetMapping(value = "/getBlock")
    public Response<BlockVO> getInfo(@RequestParam Long id) {
        return Response.succeed(this.blockService.selectBlockById(id));
    }

    /**
     * 新增区块
     */
    @PostMapping(value = "/add")
    @OperationLog(description = "新增区块", type = LogType.ADD)
    public Response<String> add(@RequestBody @Valid BlockCreateRequest blockCreateRequest) {
        this.blockService.insertBlock(blockCreateRequest);
        return Response.succeed();
    }

    /**
     * 修改区块
     */
    @PostMapping(value = "/edit")
    @OperationLog(description = "修改区块", type = LogType.UPDATE)
    public Response<String> edit(@RequestBody BlockEditRequest blockEditRequest) {
        this.blockService.updateBlock(blockEditRequest);
        return Response.succeed();
    }

    /**
     * 删除区块
     */
    @GetMapping("/delete")
    @OperationLog(description = "删除区块", type = LogType.DELETE)
    public Response<String> delete(@RequestParam Long id) {
        this.blockService.deleteBlockById(id);
        return Response.succeed();
    }

    /**
     * 查询区块列表
     */
    @GetMapping("/list")
    public Response<List<BlockVO>> list() {
        return Response.succeed(this.blockService.selectBlockList());
    }

}
