package com.isu.gaswellwatch.modbus.data.impl;

import com.isu.gaswellwatch.modbus.data.PersistenceHandler;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.PreparedStatementCallback;
import org.springframework.stereotype.Component;

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Types;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * @author <a href="mailto:scwsl@foxmail.com">王仕龙</a>
 * 2024/11/24 17:33
 */
@Component(PersistenceHandler.ETC_MODBUS_TYPE)
public class EtcPersistenceHandler extends AbstractPersistenceHandler {

    private static final Map<String, String> ETC_COLUMN_MAPPING_MAP = new HashMap<>();

    static {
        ETC_COLUMN_MAPPING_MAP.put("solenoid_valve_status", "solenoidValveStatus");
        ETC_COLUMN_MAPPING_MAP.put("controller_current_status", "controllerCurrentStatus");
        ETC_COLUMN_MAPPING_MAP.put("current_status_remaining_time", "currentStatusRemainingTime");
        ETC_COLUMN_MAPPING_MAP.put("current_status_start_time", "currentStatusStartTime");
        ETC_COLUMN_MAPPING_MAP.put("cas_pressure", "casPressure");
        ETC_COLUMN_MAPPING_MAP.put("oil_pressure", "oilPressure");
        ETC_COLUMN_MAPPING_MAP.put("pre_pressure", "prePressure");
        ETC_COLUMN_MAPPING_MAP.put("plunger_rise_time", "plungerRiseTime");
        ETC_COLUMN_MAPPING_MAP.put("arrival_sensor_delay_time", "arrivalSensorDelayTime");
        ETC_COLUMN_MAPPING_MAP.put("well_shut_in_time", "wellShutInTime");
        ETC_COLUMN_MAPPING_MAP.put("after_flow_time", "afterFlowTime");
        ETC_COLUMN_MAPPING_MAP.put("min_close_well_time", "minCloseWellTime");
        ETC_COLUMN_MAPPING_MAP.put("min_open_well_time", "minOpenWellTime");
        ETC_COLUMN_MAPPING_MAP.put("max_open_well_time", "maxOpenWellTime");
        ETC_COLUMN_MAPPING_MAP.put("max_close_well_time", "maxCloseWellTime");
        ETC_COLUMN_MAPPING_MAP.put("cas_pressure_switch", "casPressureSwitch");
        ETC_COLUMN_MAPPING_MAP.put("cas_pressure_sensor_range", "casPressureSensorRange");
        ETC_COLUMN_MAPPING_MAP.put("open_cas_pressure", "openCasPressure");
        ETC_COLUMN_MAPPING_MAP.put("open_reset_cas_pressure", "openResetCasPressure");
        ETC_COLUMN_MAPPING_MAP.put("open_cas_pressure_stable_time", "openCasPressureStableTime");
        ETC_COLUMN_MAPPING_MAP.put("close_cas_pressure", "closeCasPressure");
        ETC_COLUMN_MAPPING_MAP.put("close_trip_cas_pressure", "closeTripCasPressure");
        ETC_COLUMN_MAPPING_MAP.put("close_cas_pressure_stable_time", "closeCasPressureStableTime");
        ETC_COLUMN_MAPPING_MAP.put("tub_pressure_switch", "tubPressureSwitch");
        ETC_COLUMN_MAPPING_MAP.put("tub_pressure_sensor_range", "tubPressureSensorRange");
        ETC_COLUMN_MAPPING_MAP.put("open_tub_pressure", "openTubPressure");
        ETC_COLUMN_MAPPING_MAP.put("open_reset_tub_pressure", "openResetTubPressure");
        ETC_COLUMN_MAPPING_MAP.put("open_tub_pressure_stable_time", "openTubPressureStableTime");
    }

    @Override
    public void createTable(String tableName, Long deviceId) {
        this.createTable("sql/CREATE_ETC.sql", tableName, deviceId);
    }

    @Override
    public Map<String, Map<String, Object>> insert(String tableName, String cacheKey) {
        String insertTableSQL = this.getResource("sql/INSERT_ETC.sql");
        insertTableSQL = StringUtils.replace(insertTableSQL, "$TableName$", tableName);

        Map<String, Object> oldRow = this.getLastRow(tableName);
        Map<String, Object> newRow = this.redisTemplate.opsForHash().entries(cacheKey);

        this.jdbcTemplate.execute(insertTableSQL, new PreparedStatementCallback<Object>() {
            @Override
            public Object doInPreparedStatement(PreparedStatement ps) throws SQLException {
                ps.setLong(1, EtcPersistenceHandler.this.snowflakeConfig.snowflakeId());
                EtcPersistenceHandler.this.setValue(ps, newRow, 2, "deviceId", Types.BIGINT);
                EtcPersistenceHandler.this.setValue(ps, newRow, 3, "collectionTime", Types.TIMESTAMP);
                EtcPersistenceHandler.this.setValue(ps, newRow, 4, "receiveTime", Types.TIMESTAMP);
                EtcPersistenceHandler.this.setValue(ps, newRow, 5, "solenoidValveStatus", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, newRow, 6, "controllerCurrentStatus", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, newRow, 7, "currentStatusRemainingTime", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, newRow, 8, "currentStatusStartTime", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, newRow, 9, "casPressure", Types.DECIMAL);
                EtcPersistenceHandler.this.setValue(ps, newRow, 10, "oilPressure", Types.DECIMAL);
                EtcPersistenceHandler.this.setValue(ps, newRow, 11, "plungerRiseTime", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, newRow, 12, "arrivalSensorDelayTime", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, newRow, 13, "wellShutInTime", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, newRow, 14, "afterFlowTime", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, newRow, 15, "minCloseWellTime", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, newRow, 16, "minOpenWellTime", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, newRow, 17, "maxOpenWellTime", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, newRow, 18, "maxCloseWellTime", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, newRow, 19, "casPressureSwitch", Types.INTEGER);
                EtcPersistenceHandler.this.setValue(ps, newRow, 20, "casPressureSensorRange", Types.DECIMAL);
                EtcPersistenceHandler.this.setValue(ps, newRow, 21, "openCasPressure", Types.DECIMAL);
                EtcPersistenceHandler.this.setValue(ps, newRow, 22, "openResetCasPressure", Types.DECIMAL);
                EtcPersistenceHandler.this.setValue(ps, newRow, 23, "openCasPressureStableTime", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, newRow, 24, "closeCasPressure", Types.DECIMAL);
                EtcPersistenceHandler.this.setValue(ps, newRow, 25, "closeTripCasPressure", Types.DECIMAL);
                EtcPersistenceHandler.this.setValue(ps, newRow, 26, "closeCasPressureStableTime", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, newRow, 27, "tubPressureSwitch", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, newRow, 28, "tubPressureSensorRange", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, newRow, 29, "openTubPressure", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, newRow, 30, "openResetTubPressure", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, newRow, 31, "openTubPressureStableTime", Types.VARCHAR);
                EtcPersistenceHandler.this.setValue(ps, newRow, 32, "prePressure", Types.DECIMAL);
                return ps.executeUpdate();
            }
        });
        if (Objects.isNull(oldRow)) {
            return Map.of("new", newRow);
        }
        return Map.of("new", newRow, "old", oldRow);
    }

    @Override
    protected Map<String, String> getColumnMappingMap() {
        return ETC_COLUMN_MAPPING_MAP;
    }

}
