package com.isu.gaswellwatch.modbus.data.impl;

import com.isu.gaswellwatch.modbus.data.PersistenceHandler;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.PreparedStatementCallback;
import org.springframework.stereotype.Component;

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Types;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * @author <a href="mailto:scwsl@foxmail.com">王仕龙</a>
 * 2024/11/24 17:33
 */
@Component(PersistenceHandler.SCSS_MODBUS_TYPE)
public class ScssPersistenceHandler extends AbstractPersistenceHandler {

    private static final Map<String, String> SCSS_COLUMN_MAPPING_MAP = new HashMap<>();

    static {
        SCSS_COLUMN_MAPPING_MAP.put("cas_pressure", "casPressure");
        SCSS_COLUMN_MAPPING_MAP.put("oil_pressure", "oilPressure");
        SCSS_COLUMN_MAPPING_MAP.put("first_solenoid_status", "firstSolenoidStatus");
        SCSS_COLUMN_MAPPING_MAP.put("battery_voltage", "batteryVoltage");
        SCSS_COLUMN_MAPPING_MAP.put("solar_voltage", "solarVoltage");
        SCSS_COLUMN_MAPPING_MAP.put("remaining_time_action", "remainingTimeAction");
        SCSS_COLUMN_MAPPING_MAP.put("second_solenoid_status", "secondSolenoidStatus");
        SCSS_COLUMN_MAPPING_MAP.put("pre_pressure", "prePressure");
        SCSS_COLUMN_MAPPING_MAP.put("internet_traffic", "internetTraffic");
        SCSS_COLUMN_MAPPING_MAP.put("load_factor", "loadFactor");
        SCSS_COLUMN_MAPPING_MAP.put("data_time", "dataTime");
        SCSS_COLUMN_MAPPING_MAP.put("show_delay", "showDelay");
        SCSS_COLUMN_MAPPING_MAP.put("open_well_sampling_interval", "openWellSamplingInterval");
        SCSS_COLUMN_MAPPING_MAP.put("close_well_sampling_interval", "closeWellSamplingInterval");
        SCSS_COLUMN_MAPPING_MAP.put("ctl_model", "ctlModel");
        SCSS_COLUMN_MAPPING_MAP.put("min_pressure", "minPressure");
        SCSS_COLUMN_MAPPING_MAP.put("max_pressure", "maxPressure");
        SCSS_COLUMN_MAPPING_MAP.put("pressure_min_voltage", "pressureMinVoltage");
        SCSS_COLUMN_MAPPING_MAP.put("pressure_max_voltage", "pressureMaxVoltage");
        SCSS_COLUMN_MAPPING_MAP.put("oil_min", "oilMin");
        SCSS_COLUMN_MAPPING_MAP.put("oil_max", "oliMax");
        SCSS_COLUMN_MAPPING_MAP.put("oil_min_voltage", "oilMinVoltage");
        SCSS_COLUMN_MAPPING_MAP.put("oil_max_voltage", "oilMaxVoltage");
        SCSS_COLUMN_MAPPING_MAP.put("input_pressure_min_value", "inputPressureMinValue");
        SCSS_COLUMN_MAPPING_MAP.put("input_pressure_max_value", "inputPressureMaxValue");
        SCSS_COLUMN_MAPPING_MAP.put("input_voltage_min_value", "inputVoltageMinValue");
        SCSS_COLUMN_MAPPING_MAP.put("input_voltage_max_value", "inputVoltageMaxValue");
        SCSS_COLUMN_MAPPING_MAP.put("flow_rate_min_value", "flowRateMinValue");
        SCSS_COLUMN_MAPPING_MAP.put("flow_rate_max_value", "flowRateMaxValue");
        SCSS_COLUMN_MAPPING_MAP.put("flow_voltage_min_value", "flowVoltageMinValue");
        SCSS_COLUMN_MAPPING_MAP.put("flow_voltage_max_value", "flowVoltageMaxValue");
        SCSS_COLUMN_MAPPING_MAP.put("continuous_sampling_interval_duration", "continuousSamplingIntervalDuration");
        SCSS_COLUMN_MAPPING_MAP.put("sensor_signal_effective_level", "sensorSignalEffectiveLevel");
        SCSS_COLUMN_MAPPING_MAP.put("pressure_compensation_polarity_flag", "pressureCompensationPolarityFlag");
        SCSS_COLUMN_MAPPING_MAP.put("pressure_compensation_value_setting", "pressureCompensationValueSetting");
        SCSS_COLUMN_MAPPING_MAP.put("oil_pressure_compensation_polarity_flag", "oilPressureCompensationPolarityFlag");
        SCSS_COLUMN_MAPPING_MAP.put("oil_pressure_compensation_value_setting", "oilPressureCompensationValueSetting");
        SCSS_COLUMN_MAPPING_MAP.put("input_pressure_compensation_polarity_flag", "inputPressureCompensationPolarityFlag");
        SCSS_COLUMN_MAPPING_MAP.put("input_pressure_compensation_value_setting", "inputPressureCompensationValueSetting");
        SCSS_COLUMN_MAPPING_MAP.put("flow_compensation_polarity_flag", "flowCompensationPolarityFlag");
        SCSS_COLUMN_MAPPING_MAP.put("flow_compensation_value_setting", "flowCompensationValueSetting");
        SCSS_COLUMN_MAPPING_MAP.put("well_open_time_timestamp", "wellOpenTimeTimestamp");
        SCSS_COLUMN_MAPPING_MAP.put("well_open_pressure_value", "wellOpenPressureValue");
        SCSS_COLUMN_MAPPING_MAP.put("well_open_oil_pressure_value", "wellOpenOilPressureValue");
        SCSS_COLUMN_MAPPING_MAP.put("well_open_load_factor_presets", "wellOpenLoadFactorPresets");
        SCSS_COLUMN_MAPPING_MAP.put("well_close_time_timestamp", "wellCloseTimeTimestamp");
        SCSS_COLUMN_MAPPING_MAP.put("well_close_pressure_value", "wellClosePressureValue");
        SCSS_COLUMN_MAPPING_MAP.put("well_close_oil_pressure_value", "wellCloseOilPressureValue");
        SCSS_COLUMN_MAPPING_MAP.put("well_close_flow_value", "wellCloseFlowValue");
        SCSS_COLUMN_MAPPING_MAP.put("min_well_open_time_duration", "minWellOpenTimeDuration");
        SCSS_COLUMN_MAPPING_MAP.put("max_well_open_time_duration", "maxWellOpenTimeDuration");
        SCSS_COLUMN_MAPPING_MAP.put("min_well_close_time_duration", "minWellCloseTimeDuration");
        SCSS_COLUMN_MAPPING_MAP.put("max_well_close_time_duration", "maxWellCloseTimeDuration");
        SCSS_COLUMN_MAPPING_MAP.put("pressure_stabilization_duration", "pressureStabilizationDuration");
        SCSS_COLUMN_MAPPING_MAP.put("flow_stabilization_duration", "flowStabilizationDuration");
        SCSS_COLUMN_MAPPING_MAP.put("load_factor_stabilization_duration", "loadFactorStabilizationDuration");
        SCSS_COLUMN_MAPPING_MAP.put("plunger_delay_duration", "plungerDelayDuration");
        SCSS_COLUMN_MAPPING_MAP.put("plunger_rise_duration", "plungerRiseDuration");
        SCSS_COLUMN_MAPPING_MAP.put("continuos_flow_duration", "continuosFlowDuration");
        SCSS_COLUMN_MAPPING_MAP.put("well_close_time_duration", "wellCloseTimeDuration");
        SCSS_COLUMN_MAPPING_MAP.put("well_close_time_not_reached_duration", "wellCloseTimeNotReachedDuration");
        SCSS_COLUMN_MAPPING_MAP.put("well_close_not_reached_count_value", "wellCloseNotReachedCountValue");
        SCSS_COLUMN_MAPPING_MAP.put("plunger_delay_duration_repeat", "plungerDelayDurationRepeat");
        SCSS_COLUMN_MAPPING_MAP.put("target_time_timestamp", "targetTimeTimestamp");
        SCSS_COLUMN_MAPPING_MAP.put("target_time_range_value", "targetTimeRangeValue");
        SCSS_COLUMN_MAPPING_MAP.put("continuos_flow_increase_duration", "continuosFlowIncreaseDuration");
        SCSS_COLUMN_MAPPING_MAP.put("continuos_flow_decrease_duration", "continuosFlowDecreaseDuration");
        SCSS_COLUMN_MAPPING_MAP.put("well_close_increase_duration", "wellCloseIncreaseDuration");
        SCSS_COLUMN_MAPPING_MAP.put("well_close_decrease_duration", "wellCloseDecreaseDuration");
        SCSS_COLUMN_MAPPING_MAP.put("min_well_close_time_duration2", "minWellCloseTimeDuration2");
        SCSS_COLUMN_MAPPING_MAP.put("max_well_close_time_duration2", "maxWellCloseTimeDuration2");
        SCSS_COLUMN_MAPPING_MAP.put("min_continous_flow_time_duration", "minContinuosFlowTimeDuration");
        SCSS_COLUMN_MAPPING_MAP.put("max_continous_flow_time_duration", "maxContinuosFlowTimeDuration");
        SCSS_COLUMN_MAPPING_MAP.put("min_well_open_time_duration2", "minWellOpenTimeDuration2");
        SCSS_COLUMN_MAPPING_MAP.put("max_well_open_time_duration2", "maxWellOpenTimeDuration2");
        SCSS_COLUMN_MAPPING_MAP.put("well_open_pressure_value_at_open", "wellOpenPressureValueAtOpen");
        SCSS_COLUMN_MAPPING_MAP.put("well_open_oil_pressure_value_at_open", "wellOpenOilPressureValueAtOpen");
        SCSS_COLUMN_MAPPING_MAP.put("well_open_load_factor_presets_at_open", "wellOpenLoadFactorPresetsAtOpen");
        SCSS_COLUMN_MAPPING_MAP.put("well_close_pressure_value_at_close", "wellClosePressureValueAtClose");
        SCSS_COLUMN_MAPPING_MAP.put("well_close_oil_ppressure_value_at_close", "wellCloseOilPressureValueAtClose");
        SCSS_COLUMN_MAPPING_MAP.put("well_close_flow_value_at_close", "wellCloseFlowValueAtClose");
    }

    @Override
    public void createTable(String tableName, Long deviceId) {
        this.createTable("sql/CREATE_SCSS.sql", tableName, deviceId);
    }

    @Override
    public Map<String, Map<String, Object>> insert(String tableName, String cacheKey) {
        String insertTableSQL = this.getResource("sql/INSERT_SCSS.sql");
        insertTableSQL = StringUtils.replace(insertTableSQL, "$TableName$", tableName);

        Map<String, Object> oldRow = this.getLastRow(tableName);
        Map<String, Object> newRow = this.redisTemplate.opsForHash().entries(cacheKey);

        this.jdbcTemplate.execute(insertTableSQL, new PreparedStatementCallback<Object>() {
            @Override
            public Object doInPreparedStatement(PreparedStatement ps) throws SQLException {
                ps.setLong(1, ScssPersistenceHandler.this.snowflakeConfig.snowflakeId());
                ScssPersistenceHandler.this.setValue(ps, newRow, 2, "deviceId", Types.BIGINT);
                ScssPersistenceHandler.this.setValue(ps, newRow, 3, "collectionTime", Types.TIMESTAMP);
                ScssPersistenceHandler.this.setValue(ps, newRow, 4, "receiveTime", Types.TIMESTAMP);
                ScssPersistenceHandler.this.setValue(ps, newRow, 5, "casPressure", Types.DECIMAL);
                ScssPersistenceHandler.this.setValue(ps, newRow, 6, "oilPressure", Types.DECIMAL);
                ScssPersistenceHandler.this.setValue(ps, newRow, 7, "firstSolenoidStatus", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 8, "batteryVoltage", Types.DECIMAL);
                ScssPersistenceHandler.this.setValue(ps, newRow, 9, "solarVoltage", Types.DECIMAL);
                ScssPersistenceHandler.this.setValue(ps, newRow, 10, "remainingTimeAction", Types.VARCHAR);
                ScssPersistenceHandler.this.setValue(ps, newRow, 11, "secondSolenoidStatus", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 12, "prePressure", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 13, "internetTraffic", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 14, "loadFactor", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 15, "dataTime", Types.TIMESTAMP);
                ScssPersistenceHandler.this.setValue(ps, newRow, 16, "showDelay", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 17, "openWellSamplingInterval", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 18, "closeWellSamplingInterval", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 19, "ctlModel", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 20, "minPressure", Types.DECIMAL);
                ScssPersistenceHandler.this.setValue(ps, newRow, 21, "maxPressure", Types.DECIMAL);
                ScssPersistenceHandler.this.setValue(ps, newRow, 22, "pressureMinVoltage", Types.DECIMAL);
                ScssPersistenceHandler.this.setValue(ps, newRow, 23, "pressureMaxVoltage", Types.DECIMAL);
                ScssPersistenceHandler.this.setValue(ps, newRow, 24, "oilMin", Types.DECIMAL);
                ScssPersistenceHandler.this.setValue(ps, newRow, 25, "oliMax", Types.DECIMAL);
                ScssPersistenceHandler.this.setValue(ps, newRow, 26, "oilMinVoltage", Types.DECIMAL);
                ScssPersistenceHandler.this.setValue(ps, newRow, 27, "oilMaxVoltage", Types.DECIMAL);
                ScssPersistenceHandler.this.setValue(ps, newRow, 28, "inputPressureMinValue", Types.DECIMAL);
                ScssPersistenceHandler.this.setValue(ps, newRow, 29, "inputPressureMaxValue", Types.DECIMAL);
                ScssPersistenceHandler.this.setValue(ps, newRow, 30, "inputVoltageMinValue", Types.DECIMAL);
                ScssPersistenceHandler.this.setValue(ps, newRow, 31, "inputVoltageMaxValue", Types.DECIMAL);
                ScssPersistenceHandler.this.setValue(ps, newRow, 32, "flowRateMinValue", Types.DECIMAL);
                ScssPersistenceHandler.this.setValue(ps, newRow, 33, "flowRateMaxValue", Types.DECIMAL);
                ScssPersistenceHandler.this.setValue(ps, newRow, 34, "flowVoltageMinValue", Types.DECIMAL);
                ScssPersistenceHandler.this.setValue(ps, newRow, 35, "flowVoltageMaxValue", Types.DECIMAL);
                ScssPersistenceHandler.this.setValue(ps, newRow, 36, "continuousSamplingIntervalDuration", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 37, "sensorSignalEffectiveLevel", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 38, "pressureCompensationPolarityFlag", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 39, "pressureCompensationValueSetting", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 40, "oilPressureCompensationPolarityFlag", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 41, "oilPressureCompensationValueSetting", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 42, "inputPressureCompensationPolarityFlag", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 43, "inputPressureCompensationValueSetting", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 44, "flowCompensationPolarityFlag", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 45, "flowCompensationValueSetting", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 46, "wellOpenTimeTimestamp", Types.VARCHAR);
                ScssPersistenceHandler.this.setValue(ps, newRow, 47, "wellOpenPressureValue", Types.DECIMAL);
                ScssPersistenceHandler.this.setValue(ps, newRow, 48, "wellOpenOilPressureValue", Types.DECIMAL);
                ScssPersistenceHandler.this.setValue(ps, newRow, 49, "wellOpenLoadFactorPresets", Types.DECIMAL);
                ScssPersistenceHandler.this.setValue(ps, newRow, 50, "wellCloseTimeTimestamp", Types.VARCHAR);
                ScssPersistenceHandler.this.setValue(ps, newRow, 51, "wellClosePressureValue", Types.DECIMAL);
                ScssPersistenceHandler.this.setValue(ps, newRow, 52, "wellCloseOilPressureValue", Types.DECIMAL);
                ScssPersistenceHandler.this.setValue(ps, newRow, 53, "wellCloseFlowValue", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 54, "minWellOpenTimeDuration", Types.VARCHAR);
                ScssPersistenceHandler.this.setValue(ps, newRow, 55, "maxWellOpenTimeDuration", Types.VARCHAR);
                ScssPersistenceHandler.this.setValue(ps, newRow, 56, "minWellCloseTimeDuration", Types.VARCHAR);
                ScssPersistenceHandler.this.setValue(ps, newRow, 57, "maxWellCloseTimeDuration", Types.VARCHAR);
                ScssPersistenceHandler.this.setValue(ps, newRow, 58, "pressureStabilizationDuration", Types.VARCHAR);
                ScssPersistenceHandler.this.setValue(ps, newRow, 59, "flowStabilizationDuration", Types.VARCHAR);
                ScssPersistenceHandler.this.setValue(ps, newRow, 60, "loadFactorStabilizationDuration", Types.VARCHAR);
                ScssPersistenceHandler.this.setValue(ps, newRow, 61, "plungerDelayDuration", Types.VARCHAR);
                ScssPersistenceHandler.this.setValue(ps, newRow, 62, "plungerRiseDuration", Types.VARCHAR);
                ScssPersistenceHandler.this.setValue(ps, newRow, 63, "continuosFlowDuration", Types.VARCHAR);
                ScssPersistenceHandler.this.setValue(ps, newRow, 64, "wellCloseTimeDuration", Types.VARCHAR);
                ScssPersistenceHandler.this.setValue(ps, newRow, 65, "wellCloseTimeNotReachedDuration", Types.VARCHAR);
                ScssPersistenceHandler.this.setValue(ps, newRow, 66, "wellCloseNotReachedCountValue", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 67, "plungerDelayDurationRepeat", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 68, "targetTimeTimestamp", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 69, "targetTimeRangeValue", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 70, "continuosFlowIncreaseDuration", Types.VARCHAR);
                ScssPersistenceHandler.this.setValue(ps, newRow, 71, "continuosFlowDecreaseDuration", Types.VARCHAR);
                ScssPersistenceHandler.this.setValue(ps, newRow, 72, "wellCloseIncreaseDuration", Types.VARCHAR);
                ScssPersistenceHandler.this.setValue(ps, newRow, 73, "wellCloseDecreaseDuration", Types.VARCHAR);
                ScssPersistenceHandler.this.setValue(ps, newRow, 74, "minWellCloseTimeDuration2", Types.VARCHAR);
                ScssPersistenceHandler.this.setValue(ps, newRow, 75, "maxWellCloseTimeDuration2", Types.VARCHAR);
                ScssPersistenceHandler.this.setValue(ps, newRow, 76, "minContinuosFlowTimeDuration", Types.VARCHAR);
                ScssPersistenceHandler.this.setValue(ps, newRow, 77, "maxContinuosFlowTimeDuration", Types.VARCHAR);
                ScssPersistenceHandler.this.setValue(ps, newRow, 78, "minWellOpenTimeDuration2", Types.VARCHAR);
                ScssPersistenceHandler.this.setValue(ps, newRow, 79, "maxWellOpenTimeDuration2", Types.VARCHAR);
                ScssPersistenceHandler.this.setValue(ps, newRow, 80, "wellOpenPressureValueAtOpen", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 81, "wellOpenOilPressureValueAtOpen", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 82, "wellOpenLoadFactorPresetsAtOpen", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 83, "wellClosePressureValueAtClose", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 84, "wellCloseOilPressureValueAtClose", Types.INTEGER);
                ScssPersistenceHandler.this.setValue(ps, newRow, 85, "wellCloseFlowValueAtClose", Types.INTEGER);
                return ps.executeUpdate();
            }
        });
        if (Objects.isNull(oldRow)) {
            return Map.of("new", newRow);
        }
        return Map.of("new", newRow, "old", oldRow);
    }

    @Override
    protected Map<String, String> getColumnMappingMap() {
        return SCSS_COLUMN_MAPPING_MAP;
    }

}
