package com.isu.gaswellwatch.vo.command.etc;

import com.isu.gaswellwatch.dto.modbus.ModbusCommandDto;
import com.isu.gaswellwatch.vo.command.Command;
import com.isu.gaswellwatch.vo.command.Timing;
import lombok.Getter;
import lombok.Setter;
import lombok.experimental.SuperBuilder;
import org.apache.commons.lang3.StringUtils;

import java.io.Serial;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * @author <a href="mailto:scwsl@foxmail.com">王仕龙</a>
 * 2024/12/11 9:20
 */
@Getter
@Setter
@SuperBuilder
public class TimingMode extends Command implements Timing {

    @Serial
    private static final long serialVersionUID = -1400193679115160928L;

    /**
     * 最小关井时长
     * 100 -> 3
     */
    private String minCloseWellTime;
    /**
     * 目标上升时长
     * 106 -> 3
     */
    private String plungerRiseTime;
    /**
     * 最小开井时长
     * 121 -> 3
     */
    private String minOpenWellTime;
    /**
     * 最大开井时长
     * 130 -> 3
     */
    private String maxOpenWellTime;
    /**
     * 最大关井时长
     * 136 -> 3
     */
    private String maxCloseWellTime;
    /**
     * 柱塞未到达关井时长
     * 到达传感器延时时间
     * 139 -> 3
     */
    private String arrivalSensorDelayTime;
    /**
     * 关井时长
     * 142 -> 3
     */
    private String wellShutInTime;
    /**
     * 开井时长
     * 续流时间
     * 145 -> 3
     */
    private String afterFlowTime;

    public TimingMode() {
        this.setCode("ETC.TIMING_MODE");
    }

    @Override
    public boolean validate() {
        return super.validate()
                && StringUtils.isNotBlank(this.afterFlowTime)
                && StringUtils.isNotBlank(this.wellShutInTime)
                && StringUtils.isNotBlank(this.minOpenWellTime)
                && StringUtils.isNotBlank(this.maxOpenWellTime)
                && StringUtils.isNotBlank(this.plungerRiseTime)
                && StringUtils.isNotBlank(this.minCloseWellTime)
                && StringUtils.isNotBlank(this.maxCloseWellTime)
                && StringUtils.isNotBlank(this.arrivalSensorDelayTime);
    }

    @Override
    protected Collection<ModbusCommandDto> builderModbusCommand() {
        List<ModbusCommandDto> resultList = new ArrayList<ModbusCommandDto>(6);
        StringBuilder command = new StringBuilder(250);
        //               100
        // 地址码  功能码  起始地址  连续长度  连续字长
        // 01     10     0064    0003     06
        command.append("01100064000306");
        command.append(this.toHexString(this.minCloseWellTime));
        resultList.add(ModbusCommandDto.builder().command(command.toString()).length(16).build());

        command.setLength(0);
        //               106
        // 地址码  功能码  起始地址  连续长度  连续字长
        // 01     10     006A    0003     06
        command.append("0110006A000306");
        command.append(this.toHexString(this.plungerRiseTime));
        resultList.add(ModbusCommandDto.builder().command(command.toString()).length(16).build());

        command.setLength(0);
        //               121
        // 地址码  功能码  起始地址  连续长度  连续字长
        // 01     10     0079    0003     06
        command.append("01100079000306");
        command.append(this.toHexString(this.minOpenWellTime));
        resultList.add(ModbusCommandDto.builder().command(command.toString()).length(16).build());

        command.setLength(0);
        //               130
        // 地址码  功能码  起始地址  连续长度  连续字长
        // 01     10     0082    0003     06
        command.append("01100082000306");
        command.append(this.toHexString(this.maxOpenWellTime));
        resultList.add(ModbusCommandDto.builder().command(command.toString()).length(16).build());

        command.setLength(0);
        //               136
        // 地址码  功能码  起始地址  连续长度  连续字长
        // 01     10     0088    0006     0C
        command.append("0110008800060C");
        command.append(this.toHexString(this.maxCloseWellTime));
        command.append(this.toHexString(this.arrivalSensorDelayTime));
        resultList.add(ModbusCommandDto.builder().command(command.toString()).length(16).build());

        command.setLength(0);
        //               142
        // 地址码  功能码  起始地址  连续长度  连续字长
        // 01     10     008E    0006     0C
        command.append("0110008E00060C");
        command.append(this.toHexString(this.wellShutInTime));
        command.append(this.toHexString(this.afterFlowTime));
        resultList.add(ModbusCommandDto.builder().command(command.toString()).length(16).build());

        return resultList;
    }

    public void setMinCloseWellTime(String minCloseWellTime) {
        this.minCloseWellTime = this.timingValidate(minCloseWellTime, "minCloseWellTime");
    }

    public void setMaxOpenWellTime(String maxOpenWellTime) {
        this.maxOpenWellTime = this.timingValidate(maxOpenWellTime, "maxOpenWellTime");
    }

    public void setMinOpenWellTime(String minOpenWellTime) {
        this.minOpenWellTime = this.timingValidate(minOpenWellTime, "minOpenWellTime");
    }

    public void setMaxCloseWellTime(String maxCloseWellTime) {
        this.maxCloseWellTime = this.timingValidate(maxCloseWellTime, "maxCloseWellTime");
    }

    public void setArrivalSensorDelayTime(String arrivalSensorDelayTime) {
        this.arrivalSensorDelayTime = this.timingValidate(arrivalSensorDelayTime, "arrivalSensorDelayTime");
    }

    public void setPlungerRiseTime(String plungerRiseTime) {
        this.plungerRiseTime = this.timingValidate(plungerRiseTime, "plungerRiseTime");
    }

    public void setWellShutInTime(String wellShutInTime) {
        this.wellShutInTime = this.timingValidate(wellShutInTime, "wellShutInTime");
    }

    public void setAfterFlowTime(String afterFlowTime) {
        this.afterFlowTime = this.timingValidate(afterFlowTime, "afterFlowTime");
    }

}
